@@ -494,8 +494,6 @@ build_attr_list (ECellTextView *text_view, int row, int text_length)
 static PangoLayout *
 layout_with_preedit (ECellTextView *text_view, int row, const char *text, gint width)
 {
-	ECellView *ecell_view = (ECellView *) text_view;
-	ECellText *ect = E_CELL_TEXT (ecell_view->ecell);
 	CellEdit *edit = text_view->edit;
 	PangoAttrList *attrs ;
 	PangoLayout *layout;
@@ -559,7 +557,6 @@ build_layout (ECellTextView *text_view, int row, const char *text, gint width)
 	ECellText *ect = E_CELL_TEXT (ecell_view->ecell);
 	PangoAttrList *attrs ;
 	PangoLayout *layout;
-	gboolean bold, strikeout, underline;
 
 	layout = gtk_widget_create_pango_layout (GTK_WIDGET (((GnomeCanvasItem *)ecell_view->e_table_item_view)->canvas), text);
 
@@ -876,7 +873,6 @@ static gint
 ect_event (ECellView *ecell_view, GdkEvent *event, int model_col, int view_col, int row, ECellFlags flags, ECellActions *actions)
 {
 	ECellTextView *text_view = (ECellTextView *) ecell_view;
-	ECellText *ect = (ECellText *) ((ECellView *)text_view)->ecell;
 	ETextEventProcessorEvent e_tep_event;
 	gboolean edit_display = FALSE;
 	gint preedit_len;
@@ -1304,6 +1300,9 @@ ect_print (ECellView *ecell_view, GnomePrintContext *context,
 	GnomeFont *font = get_font_for_size (16);
 	char *string;
 	ECellText *ect = E_CELL_TEXT(ecell_view->ecell);
+	double ty, ly, text_width;
+	gboolean strikeout, underline;
+
 	string = e_cell_text_get_text(ect, ecell_view->e_table_model, model_col, row);
 	gnome_print_gsave(context);
 	if (gnome_print_moveto(context, 2, 2) == -1)
@@ -1318,8 +1317,34 @@ ect_print (ECellView *ecell_view, GnomePrintContext *context,
 				/* FIXME */;
 	if (gnome_print_clip(context) == -1)
 				/* FIXME */;
-	gnome_print_moveto(context, 2, (height - gnome_font_get_ascender(font) - gnome_font_get_descender(font)) / 2);
 
+	ty = (height - gnome_font_get_ascender(font) - gnome_font_get_descender(font)) / 2;
+	text_width = gnome_font_get_width_utf8 (font, string);
+
+	strikeout = ect->strikeout_column >= 0 && row >= 0 &&
+		e_table_model_value_at (ecell_view->e_table_model, ect->strikeout_column, row);
+	underline = ect->underline_column >= 0 && row >= 0 &&
+		e_table_model_value_at(ecell_view->e_table_model, ect->underline_column, row);
+
+	if (underline) {
+		ly = ty + gnome_font_get_underline_position (font);
+		gnome_print_newpath (context);
+		gnome_print_moveto (context, 2, ly);
+		gnome_print_lineto (context, MIN (2 + text_width, width - 2), ly);
+		gnome_print_setlinewidth (context, gnome_font_get_underline_thickness (font));
+		gnome_print_stroke (context);
+	}
+
+	if (strikeout) {
+		ly = ty + (gnome_font_get_ascender (font)  - gnome_font_get_underline_thickness (font))/ 2.0;
+		gnome_print_newpath (context);
+		gnome_print_moveto (context, 2, ly);
+		gnome_print_lineto (context, MIN (2 + text_width, width - 2), ly);
+		gnome_print_setlinewidth (context, gnome_font_get_underline_thickness (font));
+		gnome_print_stroke (context);
+	}
+
+	gnome_print_moveto(context, 2, ty);
 	gnome_print_setfont(context, font);
 	gnome_print_show(context, string);
 	gnome_print_grestore(context);
@@ -1677,28 +1702,28 @@ e_cell_text_class_init (GObjectClass *object_class)
 					 g_param_spec_int ("strikeout_column",
 							   _("Strikeout Column"),
 							   /*_( */"XXX blurb" /*)*/,
-							   0, G_MAXINT, 0,
+							   -1, G_MAXINT, -1,
 							   G_PARAM_READWRITE));
 
 	g_object_class_install_property (object_class, PROP_UNDERLINE_COLUMN,
 					 g_param_spec_int ("underline_column",
 							   _("Underline Column"),
 							   /*_( */"XXX blurb" /*)*/,
-							   0, G_MAXINT, 0,
+							   -1, G_MAXINT, -1,
 							   G_PARAM_READWRITE));
 
 	g_object_class_install_property (object_class, PROP_BOLD_COLUMN,
 					 g_param_spec_int ("bold_column",
 							   _("Bold Column"),
 							   /*_( */"XXX blurb" /*)*/,
-							   0, G_MAXINT, 0,
+							   -1, G_MAXINT, -1,
 							   G_PARAM_READWRITE));
 
 	g_object_class_install_property (object_class, PROP_COLOR_COLUMN,
 					 g_param_spec_int ("color_column",
 							   _("Color Column"),
 							   /*_( */"XXX blurb" /*)*/,
-							   0, G_MAXINT, 0,
+							   -1, G_MAXINT, -1,
 							   G_PARAM_READWRITE));
 
 	g_object_class_install_property (object_class, PROP_EDITABLE,
@@ -1712,7 +1737,7 @@ e_cell_text_class_init (GObjectClass *object_class)
 					 g_param_spec_int ("bg_color_column",
 							   _("BG Color Column"),
 							   /*_( */"XXX blurb" /*)*/,
-							   0, G_MAXINT, 0,
+							   -1, G_MAXINT, -1,
 							   G_PARAM_READWRITE));
 
 	if (!clipboard_atom)
@@ -1755,7 +1780,6 @@ e_cell_text_commit_cb (GtkIMContext *context,
 		  ECellTextView    *tv)
 {
 	CellEdit *edit = tv->edit;
-	ECellText *ect = (ECellText *) ((ECellView *)tv)->ecell;
 	ETextEventProcessorCommand command;
 	                                                  
 	if (g_utf8_validate (str, strlen (str), NULL)) {
